/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util.actions;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import org.openide.util.actions.CookieAction;
import org.openide.nodes.Node;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * This class provides a simplified way to work with {@link CookieAction}. You have to pass the classes describing the
 * handled capabilities in the constructor, and override a single method {@link #performAction(java.util.List)} whose
 * parameter is a list of the capabilities upon which the action must be performed. In other words, this class does
 * on its own the lookup operation on {@link Node}s, so you don't deal with them. Also the enablement control can be
 * directly made on single capability instances, by overriding {@link #enable(java.lang.Object)}.
 *
 * <pre>
 * public class Stuff
 *   {
 *     // ...
 *   }
 *
 * public class StuffAction extends GenericCapabilityAction<Stuff>
 *   {
 *     public StuffAction()
 *       {
 *         super(Stuff.class);
 *       }
 *
 *     @Override
 *     public void performAction (final @Nonnull List<Stuff> capabilities)
 *       {
 *         for (final Stuff stuff : capabilities)
 *           {
 *             // do something with stuff
 *           }
 *       }
 * 
 *     @Override
 *     protected int mode() // optional if the default MODE_ANY is ok
 *       {
 *         return CookieAction.MODE_ALL;
 *       }
 *
 *     @Override
 *     protected boolean enable (final @Nonnull T capability) // optional
 *       {
 *         // return false if capability can't be worked on
 *       }
 *   }
 * </pre>
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class GenericCapabilityAction<T> extends EnhancedCapabilityAction
  {
    private static final String CLASS = GenericCapabilityAction.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    protected final Class<? extends T>[] capabilityClasses;

    /*******************************************************************************************************************
     *
     * Creates an instance of this action that will operate on instances of the given capability classes.
     *
     * @param  capabilityClasses   the classes of capabilities
     *
     ******************************************************************************************************************/
    protected GenericCapabilityAction (final @Nonnull Class<? extends T> ... capabilityClasses)
      {
        if (capabilityClasses.length == 0)
          {
            throw new IllegalArgumentException("No cookie classes specified");
          }

        this.capabilityClasses = capabilityClasses.clone();
      }

    /*******************************************************************************************************************
     *
     * This method is called when the action has been triggered. The arguments are the capabilities handled by this
     * action.
     *
     * @param  capabilities  the capabilities handled by this action
     *
     ******************************************************************************************************************/
    protected abstract void performAction (@Nonnull List<T> capabilities)
      throws Exception;

    /*******************************************************************************************************************
     *
     * This method controls the enablement status of this action. It should return {@code false} if the passed
     * capability can't be worked on. The default implementation always returns {@code true}.
     *
     * @param  capability   the capability to test
     * @return              false if the capability can't be worked on
     *
     ******************************************************************************************************************/
    protected boolean enable (final @Nonnull T capability)
      {
        return true;
      }

    /*******************************************************************************************************************
     *
     * Returns the classes of capabilities handled by this action.
     *
     * @return  the capability classes
     *
     ******************************************************************************************************************/
    @Nonnull
    public final Class<? extends T>[] getCapabilityClasses()
      {
        return capabilityClasses.clone();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected int mode()
      {
        return CookieAction.MODE_ANY;
      }

//    /*******************************************************************************************************************
//     *
//     * {@inheritDoc}
//     *
//     ******************************************************************************************************************/
//    @Override
//    protected void initialize()
//      {
//        super.initialize();
////        putValue("noIconInMenu", Boolean.TRUE);
//      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public final void performAction (final @Nonnull Node[] nodes)
      {
        try
          {
            performAction(getCapabilities(nodes));
          }
        catch (Exception e)
          {
            logger.warning("Exception in performAction(): %s", e);
            logger.throwing(CLASS, "performAction()", e);
            e.printStackTrace();
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected boolean enable (final @Nonnull Node[] nodes)
      {
        logger.fine("enable(%d nodes) - %s", nodes.length, getClass().getName());
        logger.finest(">>>> nodes: %s", Arrays.toString(nodes));

        if (!super.enable(nodes))
          {
            logger.finer(">>>> super() returned false");
            return false;
          }

        for (final T capability : getCapabilities(nodes))
          {
            if ((capability != null) && !enable(capability))
              {
                logger.finer(">>>> enable(%s) returning false", capability);
                return false;
              }
          }

        logger.finer(">>>> enable() returning true");
        return true;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    protected final Class[] cookieClasses() // implementation for CookieAction
      {
        return getCapabilityClasses();
      }

    /*******************************************************************************************************************
     *
     * Extracts the capabilities out of an array of {@link Nodes}.
     *
     * @param  nodes   the {@link Node}s
     * @return         the capabilities
     *
     ******************************************************************************************************************/
    @Nonnull
    protected final List<T> getCapabilities (final @Nonnull Node[] nodes)
      {
        final List<T> capabilities = new ArrayList<T>();

        for (final Node node : nodes)
          {
            for (final Class<? extends T> capabilityClass : capabilityClasses)
              {
                final T capability = node.getLookup().lookup(capabilityClass);

                if (capability != null)
                  {
                    capabilities.add(capability);
                  }
              }
          }
        
        return capabilities;
      }
  }
